#include "configuration.h"

/*BLE DFU*/
#include <zephyr/mgmt/mcumgr/mgmt/mgmt.h>
#include <zephyr/mgmt/mcumgr/mgmt/callbacks.h>
#include <zephyr/settings/settings.h>
#include <zephyr/logging/log.h>

#include "DFU.h"
#include "AEI.h"

LOG_MODULE_REGISTER(DFU);


/*BLE DFU authorisation callback*/
struct mgmt_callback struct_dfu_authorisation_callback;

enum mgmt_cb_return cb_dfu_authorisation_callback(uint32_t event, enum mgmt_cb_return prev_status,
                                int32_t *rc, uint16_t *group, bool *abort_more,
                                void *data, size_t data_size)
{
	if (event == MGMT_EVT_OP_IMG_MGMT_DFU_CHUNK && !USB_STATUS)
	{
		LOG_WRN ("DFU command disallowed");
		return MGMT_ERR_EBADSTATE;
	}

	LOG_INF ("DFU command authorized") ;
	return MGMT_CB_OK;
}


/*BLE DFU status callback*/
struct mgmt_callback struct_dfu_status_callback;

enum mgmt_cb_return cb_dfu_status_callback(uint32_t event, enum mgmt_cb_return prev_status,
                                int32_t *rc, uint16_t *group, bool *abort_more,
                                void *data, size_t data_size)
{
    if (event == MGMT_EVT_OP_IMG_MGMT_DFU_STARTED) {
        LOG_INF ("Upgarde started");
		*value_status_ble = BLE_DFU_STARTED;
		send_trame_async(ASYN_BLE_STATUS,value_status_ble,LEN_STRING_STATUS_RADIO_MODULE);
		SERVICE_BIDIR_EN = 0;
    }

	if (event == MGMT_EVT_OP_IMG_MGMT_DFU_STOPPED) {
		LOG_INF("Upgrade stopped");
		*value_status_ble = BLE_DFU_STOPPED;
		send_trame_async(ASYN_BLE_STATUS,value_status_ble,LEN_STRING_STATUS_RADIO_MODULE);
		SERVICE_BIDIR_EN = 1;
	}

	if (event == MGMT_EVT_OP_IMG_MGMT_DFU_PENDING) {
		LOG_INF("Upgrade finished");
		*value_status_ble = BLE_DFU_FINISHED;
		send_trame_async(ASYN_BLE_STATUS,value_status_ble,LEN_STRING_STATUS_RADIO_MODULE);
		SERVICE_BIDIR_EN = 1;
	}

	if (event == MGMT_EVT_OP_IMG_MGMT_DFU_CONFIRMED) {
		LOG_INF("New firmware version confirmed");
		*value_status_ble = BLE_DFU_CONFIRMED;
		send_trame_async(ASYN_BLE_STATUS,value_status_ble,LEN_STRING_STATUS_RADIO_MODULE);
	}

    /* Return OK status code to continue with acceptance to underlying handler */
    return MGMT_CB_OK;
}

/*Access to settings callback*/
struct mgmt_callback settings_access_struct;

enum mgmt_cb_return settings_access_cb(uint32_t event, enum mgmt_cb_return prev_status,
                                int32_t *rc, uint16_t *group, bool *abort_more,
                                void *data, size_t data_size)
{
	LOG_INF("Access to settings attempted and allowed");
    /* Return OK status code to continue with acceptance to underlying handler */
    return MGMT_CB_OK;
}

void init_dfu (void)
{
	struct_dfu_authorisation_callback.callback = cb_dfu_authorisation_callback;
	struct_dfu_authorisation_callback.event_id = (MGMT_EVT_OP_IMG_MGMT_DFU_CHUNK);
	mgmt_callback_register(&struct_dfu_authorisation_callback);

    struct_dfu_status_callback.callback = cb_dfu_status_callback;
	struct_dfu_status_callback.event_id = (MGMT_EVT_OP_IMG_MGMT_DFU_STARTED | MGMT_EVT_OP_IMG_MGMT_DFU_STOPPED 
										 | MGMT_EVT_OP_IMG_MGMT_DFU_PENDING | MGMT_EVT_OP_IMG_MGMT_DFU_CONFIRMED);
	mgmt_callback_register(&struct_dfu_status_callback);

	settings_access_struct.callback = settings_access_cb;
	settings_access_struct.event_id = (MGMT_EVT_OP_SETTINGS_MGMT_ALL );
	mgmt_callback_register(&settings_access_struct);

	LOG_INF("Mcumgr callbacks added for DFU events");
}